Package: Gmisc
Version: 3.4.0
Title: Descriptive Statistics, Transition Plots, and More
Authors@R: c(
    person(given = "Max", family = "Gordon",
           email = "max@gforge.se", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-8080-5815")))
Maintainer: Max Gordon <max@gforge.se>
Description: Utilities for common medical-statistics graphics
    and tables, including automatic creation of publication-ready "Table 1"
    summaries; transition (Sankey) plots; flow-chart helpers that extend the
    grid package; Bézier lines and arrows; and a singular-value-decomposition-
    based variable-selection method.
License: GPL (>= 3)
URL: https://gforge.se
BugReports: https://github.com/gforge/Gmisc/issues
Depends: R (>= 4.1.0), htmlTable (>= 2.0.0)
Imports: Rcpp (>= 0.11.4), abind, checkmate, forestplot, Hmisc, glue,
        grid, grDevices, graphics, knitr, lattice, lubridate, magrittr,
        methods, rlang, rmarkdown, stringr, stats, XML, yaml, utils
Suggests: bench, vdiffr, datasets, dplyr, jsonlite, testthat,
        tidyselect, webshot2
Encoding: UTF-8
NeedsCompilation: yes
LinkingTo: Rcpp
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
Packaged: 2026-06-20 10:49:57 UTC; max
Author: Max Gordon [aut, cre] (ORCID: <https://orcid.org/0000-0002-8080-5815>)
Repository: CRAN
Date/Publication: 2026-06-20 11:30:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-06-20 15:21:16 UTC; unix
Archs: Gmisc.so.dSYM
