# Generated by roxygen2: do not edit by hand

S3method(align,Gmisc_list_of_boxes)
S3method(align,default)
S3method(align,list)
S3method(append,Gmisc_list_of_boxes)
S3method(append,default)
S3method(connect,Gmisc_list_of_boxes)
S3method(connect,default)
S3method(getDescriptionStatsBy,data.frame)
S3method(getDescriptionStatsBy,default)
S3method(grid.draw,connect_boxes_list)
S3method(heightDetails,box)
S3method(htmlTable,Gmisc_getDescriptionStatsBy)
S3method(htmlTable,descMrg)
S3method(insert,Gmisc_list_of_boxes)
S3method(insert,default)
S3method(knit_print,Gmisc_getDescriptionStatsBy)
S3method(knit_print,descMrg)
S3method(latex,descMrg)
S3method(length,Gmisc_getDescriptionStatsBy)
S3method(move,Gmisc_list_of_boxes)
S3method(move,default)
S3method(phaseLabel,Gmisc_list_of_boxes)
S3method(phaseLabel,default)
S3method(plot,box)
S3method(plot,connect_boxes)
S3method(plot,connect_boxes_list)
S3method(position,box)
S3method(position,coords)
S3method(position,default)
S3method(prCalculateConnector,ConnectorStrategy)
S3method(prCalculateConnector,ManyToOneFanInCenterConnectorStrategy)
S3method(prCalculateConnector,ManyToOneFanInTopConnectorStrategy)
S3method(prCalculateConnector,ManyToOneNConnectorStrategy)
S3method(prCalculateConnector,OneToManyNConnectorStrategy)
S3method(print,Gmisc_connector_label)
S3method(print,Gmisc_extended_connector_labels)
S3method(print,Gmisc_getDescriptionStatsBy)
S3method(print,Gmisc_list_of_boxes)
S3method(print,Gmisc_unit)
S3method(print,box)
S3method(print,connect_boxes)
S3method(print,connect_boxes_list)
S3method(print,descMrg)
S3method(spread,Gmisc_list_of_boxes)
S3method(spread,default)
S3method(spread,list)
S3method(widthDetails,box)
S3method(yamlDump,character)
S3method(yamlDump,default)
S3method(yamlDump,json)
export(MidDistanceX)
export(MidDistanceY)
export(align)
export(alignHorizontal)
export(alignVertical)
export(append)
export(bezierArrowGradient)
export(bezierArrowSmpl)
export(boxDatabaseGrob)
export(boxDiamondGrob)
export(boxDocumentGrob)
export(boxDocumentsGrob)
export(boxEllipseGrob)
export(boxGrob)
export(boxHeaderGrob)
export(boxPropGrob)
export(boxRackGrob)
export(boxServerGrob)
export(boxTapeGrob)
export(connect)
export(connectGrob)
export(convertShowMissing)
export(coords)
export(copyAllNewAttributes)
export(database_box_fn)
export(descGetMissing)
export(describeFactors)
export(describeMean)
export(describeMedian)
export(describeProp)
export(diamond_rounded_box_fn)
export(diamond_sharp_box_fn)
export(distance)
export(document_box_fn)
export(documents_box_fn)
export(docx_document)
export(ellipse_box_fn)
export(equalizeHeights)
export(equalizeWidths)
export(fastDoCall)
export(figCapNo)
export(figCapNoLast)
export(figCapNoNext)
export(flowchart)
export(forestplot2)
export(getDescriptionStatsBy)
export(getPvalAnova)
export(getPvalChiSq)
export(getPvalFisher)
export(getPvalKruskal)
export(getPvalWilcox)
export(getSvdMostInfluential)
export(gnrlBezierPoints)
export(has)
export(insert)
export(insertRowAndKeepAttr)
export(labelConnector)
export(mergeDesc)
export(mergeLists)
export(move)
export(moveBox)
export(pathJoin)
export(phaseLabel)
export(position)
export(rack_box_fn)
export(retrieve)
export(server_box_fn)
export(setConnectorLabels)
export(set_column_labels)
export(set_column_units)
export(spread)
export(spreadHorizontal)
export(spreadVertical)
export(tape_box_fn)
export(time2spanTxt)
export(transitionPlot)
export(yamlDump)
exportClasses(Transition)
import(XML)
import(abind)
import(forestplot)
import(grid)
import(htmlTable)
import(lattice)
import(magrittr)
importFrom(Hmisc,"label<-")
importFrom(Hmisc,capitalize)
importFrom(Hmisc,label)
importFrom(Hmisc,latex)
importFrom(Hmisc,units)
importFrom(Rcpp,evalCpp)
importFrom(checkmate,assert)
importFrom(checkmate,assert_class)
importFrom(checkmate,assert_list)
importFrom(checkmate,assert_number)
importFrom(checkmate,checkCharacter)
importFrom(checkmate,checkClass)
importFrom(checkmate,checkList)
importFrom(checkmate,checkNumeric)
importFrom(checkmate,checkString)
importFrom(checkmate,checkTRUE)
importFrom(glue,glue)
importFrom(grDevices,as.raster)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,grey)
importFrom(grDevices,rgb)
importFrom(graphics,par)
importFrom(grid,gpar)
importFrom(grid,grid.draw)
importFrom(grid,is.unit)
importFrom(knitr,asis_output)
importFrom(knitr,knit_print)
importFrom(lubridate,as_date)
importFrom(lubridate,mday)
importFrom(lubridate,month)
importFrom(lubridate,year)
importFrom(methods,formalArgs)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,setRefClass)
importFrom(rlang,enquo)
importFrom(rlang,expr)
importFrom(rmarkdown,html_document)
importFrom(stats,anova)
importFrom(stats,chisq.test)
importFrom(stats,fisher.test)
importFrom(stats,kruskal.test)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,wilcox.test)
importFrom(stringr,str_detect)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_trim)
importFrom(utils,as.roman)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(yaml,as.yaml)
useDynLib(Gmisc)
